@@ -25,17 +25,18 @@
 #include <X11/Xutil.h>
 #include <X11/xpm.h>
 #include <X11/extensions/shape.h>
+#include <X11/cursorfont.h>
 
 #include "wmbluecpu.h"
 #include "options.h"
 #include "draw.h"
 #include "timing.h"
 
-#include "bg.xpm"
-#include "on.xpm"
-#include "off.xpm"
-#include "numbers.xpm"
-#include "panel.xpm"
+#include "xpm/bg.xpm"
+#include "xpm/on.xpm"
+#include "xpm/off.xpm"
+#include "xpm/numbers.xpm"
+#include "xpm/panel.xpm"
 
 #define WINDOW_WIDTH 64
 #define WINDOW_HEIGHT 64
@@ -47,6 +48,10 @@ Colormap colormap;
 GC       gc;
 Pixmap   background, backgroundmask, on, off, numbers, buffer, panel;
 Atom     wm_delete_window;
+Atom     _motif_wm_hints;
+int      moving, oldx, oldy;
+int      screenwidth, screenheight;
+Cursor   fleur;
 
 void free_stuff()
 {
@@ -94,6 +99,14 @@ void make_window()
  XSizeHints           *sizehints;
  XGCValues             gcvalues;
  unsigned int          depth;
+ struct
+ {
+  long flags;
+  long functions;
+  long decorations;
+  long input_mode;
+  long unknown;
+ } mwmhints;
 
  display = XOpenDisplay(opt_display);
  if(!display)
@@ -102,6 +115,8 @@ void make_window()
   exit(1);
  }
  screen = DefaultScreen(display);
+ screenwidth = DisplayWidth(display, screen);
+ screenheight = DisplayHeight(display, screen);
  rootwindow = RootWindow(display, screen);
  colormap = DefaultColormap(display, screen);
  depth = DefaultDepth(display, screen);
@@ -119,7 +134,11 @@ void make_window()
  buffer = XCreatePixmap(display, rootwindow,
                         WINDOW_WIDTH, WINDOW_HEIGHT, depth);
  windowattributes.background_pixmap = background;
- windowattributes.event_mask = ExposureMask | ButtonPressMask | PropertyChangeMask;
+ windowattributes.event_mask = ExposureMask |
+                               ButtonPressMask |
+			       ButtonReleaseMask |
+			       PointerMotionMask |
+			       PropertyChangeMask;
  windowattributes.colormap = colormap;
  window = XCreateWindow(display, rootwindow,
                         0, 0, WINDOW_WIDTH, WINDOW_HEIGHT, 0,
@@ -135,6 +154,7 @@ void make_window()
 			    CopyFromParent,
 			    CWBackPixmap | CWEventMask,
 			    &windowattributes);
+ fleur = XCreateFontCursor(display, XC_fleur);
  if(XShapeQueryExtension(display, &shapeevent, &shapeerror) && opt_shape)
  {
   XShapeCombineMask(display, window, ShapeBounding,
@@ -195,7 +215,8 @@ void make_window()
  XSetWMNormalHints(display, mapwindow, sizehints);
  XFree(sizehints);
 
- XStoreName(display, mapwindow, OPT_WINDOW_NAME);
+ XStoreName(display, window, OPT_WINDOW_NAME);
+ XStoreName(display, iconwindow, OPT_WINDOW_NAME); /* For other wms */
 
  gcvalues.graphics_exposures = False;
  gcvalues.function = GXcopy;
@@ -206,6 +227,16 @@ void make_window()
  XSetCommand(display, window, argv, argc);
  wm_delete_window = XInternAtom(display, "WM_DELETE_WINDOW", 0);
  XSetWMProtocols(display, mapwindow, &wm_delete_window, 1);
+ if(opt_window)
+ {
+  _motif_wm_hints = XInternAtom(display, "_MOTIF_WM_HINTS", 0);
+  mwmhints.flags = 2;
+  mwmhints.functions = 0x71; /* WTF IS THIS ? */
+  mwmhints.decorations = 0;
+  XChangeProperty(display, mapwindow,
+                  _motif_wm_hints, _motif_wm_hints,
+		  32, PropModeReplace, (unsigned char *)&mwmhints, 5);
+ }
  XMapWindow(display, mapwindow);
  signal(SIGINT, handle_signal);
  signal(SIGQUIT, handle_signal);
@@ -221,6 +252,7 @@ void update_window()
 void process_events()
 {
  XEvent event;
+ int winx, winy;
 
  XNextEvent(display, &event);
  switch(event.type)
@@ -229,6 +261,30 @@ void process_events()
    update_window();
    break;
   case ButtonPress:
+   if(opt_window && (event.xbutton.button == 1))
+   {
+    XDefineCursor(display, mapwindow, fleur);
+    moving = 1;
+    oldx = event.xbutton.x;
+    oldy = event.xbutton.y;
+   }
+   break;
+  case MotionNotify:
+   winx = event.xmotion.x_root - oldx;
+   winy = event.xmotion.y_root - oldy;
+   if(winx < 0) winx = 0;
+   if(winy < 0) winy = 0;
+   if(winx > (screenwidth - WINDOW_WIDTH)) winx = screenwidth - WINDOW_WIDTH;
+   if(winy > (screenheight - WINDOW_HEIGHT)) winy = screenheight - WINDOW_HEIGHT;
+   if(moving)
+    XMoveWindow(display, mapwindow, winx, winy);
+   break;
+  case ButtonRelease:
+   if(opt_window)
+   {
+    moving = 0;
+    XUndefineCursor(display, mapwindow);
+   }
    break;
   case ClientMessage:
    if(event.xclient.data.l[0] == wm_delete_window)
