@@ -24,8 +24,6 @@
 #include <X11/Xlib.h>
 
 extern Display *display;
-extern int      screen;
-extern Window   window, iconwindow;
 extern GC       gc;
 extern Pixmap   background, backgroundmask, on, off, numbers, buffer;
 
